package io.nisum.designpattern.employee.service.impl;

import io.nisum.designpattern.employee.dto.EmployeeDto;
import io.nisum.designpattern.employee.dto.EmployeePatchDto;
import io.nisum.designpattern.employee.mapper.MapperModel;
import io.nisum.designpattern.employee.repo.EmployeeRepository;
import io.nisum.designpattern.employee.service.EmployeeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

@Service
public class EmployeeServiceImpl implements EmployeeService {

    @Autowired
    EmployeeRepository employeeRepository;

    @Override
    public Flux<EmployeeDto> findAll() {
        return employeeRepository.findAll().map(MapperModel::mapToDto);
    }

    @Override
    public Mono<EmployeeDto> findById(long id) {
        return employeeRepository.findById(id).map(MapperModel::mapToDto);
    }

    @Override
    public Mono<EmployeeDto> findByName(String name) {
        return employeeRepository.findByName(name).map(MapperModel::mapToDto);
    }

    @Override
    public Flux<EmployeeDto> findByDepartment(long departmentId) {
        return employeeRepository.findByDepartmentId(departmentId).map(MapperModel::mapToDto);
    }

    @Override
    public Flux<EmployeeDto> findByOrganization(long organizationId) {
        return employeeRepository.findByOrganizationId(organizationId).map(MapperModel::mapToDto);
    }

    @Override
    public Mono<EmployeeDto> save(Mono<EmployeeDto> employeeDto) {
        return employeeDto
                .map(MapperModel::mapToEntity)
                .flatMap(employeeRepository::save)
                .map(MapperModel::mapToDto);
    }

    @Override
    public Mono<EmployeeDto> update(long id, Mono<EmployeeDto> employeeDto) {
        return employeeRepository
                .findById(id)
                .flatMap(e -> employeeDto.map(MapperModel::mapToEntity))
                .doOnNext(e -> e.setId(id))
                .flatMap(employeeRepository::save)
                .map(MapperModel::mapToDto);
    }

    @Override
    public Mono<EmployeeDto> patch(long id, Mono<EmployeePatchDto> employeePatchDto) {
        return employeeRepository
                .findById(id)
                .zipWith( employeePatchDto, (emp, empDto) -> {
                    return MapperModel.patchEmployee( empDto, emp );
                } )
                .flatMap(employeeRepository::save)
                .map(MapperModel::mapToDto);
    }

    @Override
    public Mono<Void> delete(long id) {
        return employeeRepository.deleteById(id);
    }

}
