package io.nisum.designpattern.organization.service.impl;

import io.nisum.designpattern.organization.dto.OrganizationDto;
import io.nisum.designpattern.organization.mapper.MapperModel;
import io.nisum.designpattern.organization.repo.OrganizationRepo;
import io.nisum.designpattern.organization.service.OrganizationService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

@Service
public class OrganizationServiceImpl implements OrganizationService {

    @Autowired
    OrganizationRepo organizationRepo;

    @Override
    public Mono<OrganizationDto> save(Mono<OrganizationDto> organizationDtoMono) {
        return organizationDtoMono
                .map(MapperModel::mapToEntity)
                .flatMap(organizationRepo::save)
                .map(MapperModel::mapToDto);
    }

    @Override
    public Flux<OrganizationDto> findAll() {
        return organizationRepo
                .findAll()
                .map(MapperModel::mapToDto);
    }

    @Override
    public Mono<OrganizationDto> findById(long id) {
        return organizationRepo
                .findById(id)
                .map(MapperModel::mapToDto);
    }
}
